<?php
declare(strict_types=1);
session_start();

require __DIR__ . '/vendor/autoload.php';

use Twig\Loader\FilesystemLoader;
use Twig\Environment;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Symfony\Component\Dotenv\Dotenv;
use Curl\Curl;

//Loading config file
$dotenv = new Dotenv();
$dotenv->load(__DIR__ . '/.env');

//Loading TWIG
$loader = new FilesystemLoader('templates/'); // Dossier contenant les templates
$twig = new Environment($loader, array(
    'cache' => false,
    'debug' => true
));

$request = \Laminas\Diactoros\ServerRequestFactory::fromGlobals($_SERVER, $_GET, $_POST, $_COOKIE, $_FILES);

$router = new League\Route\Router();


/**
 *
 * VERSIONING
 *
 */
$router->map('GET', '/', function (ServerRequestInterface $request) use ($twig) : ResponseInterface {
    $response = new Laminas\Diactoros\Response;
    $response->getBody()->write(
        $twig->render('home.html.twig', [
            'acceptHeader' => $request->getHeader('Accept')[0]

        ])
    );
    return $response;
});
##API_VERSIONING##
$router->map('POST', '/ajax-versioning', function (ServerRequestInterface $request) use ($twig) : ResponseInterface {
    $curl = new Curl();
    $curl->setBasicAuthentication($_ENV["API_USER"], $_ENV["API_PASSWORD"]);

    $curl->setHeader('Content-Type', 'application/x-www-form-urlencoded');
    $postVars['acctNumber'] = $_POST["card"]["cardNumber"];
    $curl->post($_ENV["HOST_CENTRALPAY_API_CORE"] . '3ds2/versioning', $postVars);

    if ($curl->error) {
        return new Laminas\Diactoros\Response\JsonResponse(['success' => false,'error'=>$curl->curl_error_code,'msg'=>$curl->curl_error_message]);
    } else {
        $content = json_decode($curl->getResponse(), true);
        $_SESSION['versioning'] = $content;
        $_SESSION['card'] = $_POST;
        return new Laminas\Diactoros\Response\JsonResponse(['success' => true]);
    }


});

/**
 *
 * AUTHENTIFICATION
 *
 */
##API_AUTHENTICATION##
$router->map('POST', '/ajax-valid-authentification', function (ServerRequestInterface $request) use ($twig) : ResponseInterface {

    $postVars = $_POST["data"];
    $postVars["threeDSServerTransID"] = $_SESSION['versioning']["threeDSServerTransID"];
    $postVars["acctNumber"] = $_SESSION["card"]['card']['cardNumber'];
    $postVars["deviceChannel"] = '02';
    $postVars["messageCategory"] = '01';
    $postVars["purchaseAmount"] = round($_SESSION["card"]['card']['amount'] * 100, 0);
    $postVars["purchaseCurrency"] = 'EUR';
    $postVars["threeDSRequestorAuthenticationInd"] = '01';
    $postVars["notificationURL"] = $request->getServerParams()['REQUEST_SCHEME'] . '://' . $request->getServerParams()['HTTP_HOST'] . '/challenge/back';
    $postVars["threeDSRequestorURL"] = 'https://www.centralpay.eu';
    $curl = new Curl();
    $curl->setBasicAuthentication($_ENV["API_USER"], $_ENV["API_PASSWORD"]);
    $curl->setHeader('Content-Type', 'application/x-www-form-urlencoded');
    $curl->post($_ENV["HOST_CENTRALPAY_API_CORE"] . '3ds2/authentication', $postVars);
    if ($curl->error) {
        return new Laminas\Diactoros\Response\JsonResponse(['success' => false]);
    } else {
        $content = json_decode($curl->getResponse(), true);
        $_SESSION['authentification'] = $content;
        return new Laminas\Diactoros\Response\JsonResponse($content);
    }


});

/**
 *
 * 3DS METHOD
 *
 */
$router->map('GET', '/iframe-3ds-method', function (ServerRequestInterface $request) use ($twig) : ResponseInterface {
    $response = new Laminas\Diactoros\Response;
    $response->getBody()->write(
        $twig->render('3ds-method.html.twig', [
            'versioning' => $_SESSION["versioning"]

        ])
    );

    return $response;
});

/**
 *
 * CHALLENGE
 *
 */

$router->map('GET', '/iframe-post-challenge', function (ServerRequestInterface $request) use ($twig) : ResponseInterface {

    $response = new Laminas\Diactoros\Response;
    $response->getBody()->write(
        $twig->render('challenge.html.twig', [
            'versioning' => $_SESSION["versioning"],
            'authentification' => $_SESSION["authentification"]

        ])
    );

    return $response;
});

$router->map('GET', '/post-challenge', function (ServerRequestInterface $request) use ($twig) : ResponseInterface {

    $response = new Laminas\Diactoros\Response;
    $response->getBody()->write(
        $twig->render('challenge-post.html.twig', [
            'versioning' => $_SESSION["versioning"],
            'authentification' => $_SESSION["authentification"]

        ])
    );
    return $response;
});


$router->map('POST', '/challenge/back', function (ServerRequestInterface $request) use ($twig) : ResponseInterface {

    if ($_ENV["APP_ENV"] == 'dev') {
        $ipClient = '192.192.192.192';
    } else {
        $ipClient = $request->getAttribute('REMOTE_ADDR');
    }

    $retour = json_decode(base64_decode($_POST['cres']), true);

    $_SESSION['errorOtp'] = false;
    if (in_array($retour['transStatus'], ['Y', 'A'])) {
        ##API_3DS2_RESULTS##
        $curl = new Curl();
        $curl->setBasicAuthentication($_ENV["API_USER"], $_ENV["API_PASSWORD"]);
        $curl->setHeader('Content-Type', 'application/x-www-form-urlencoded');

        $curl->get($_ENV["HOST_CENTRALPAY_API_CORE"] . '3ds2/results/'.$retour['threeDSServerTransID']);

        $retourOtp = json_decode($curl->getResponse(), true);

        $_SESSION['retourOtp'] = $retourOtp;
        $curl->close();
        ##API_TRANSACTION##
        $postVars = [
            'currency' => 'EUR',
            'amount' => round($_SESSION['card']['card']['amount'] * 100, 0),
            'endUserIp' => $ipClient,
            'merchantTransactionId' => 'REFERENCE XXX',
            'pointOfSaleId' => $_ENV["POS_UUID"],
            'browserUserAgent' => $request->getHeader('User-Agent'),
            'browserAcceptLanguage' => 'fre',
        ];
        $otp = $_SESSION['retourOtp'];
        $postVars['card[number]'] = $_SESSION['card']['card']['cardNumber'];
        $postVars['card[cvc]'] = $_SESSION['card']['card']['cvc'];
        $postVars['card[expirationMonth]'] = $_SESSION['card']['card']['dateexpiration']['month'];
        $postVars['card[expirationYear]'] = $_SESSION['card']['card']['dateexpiration']['year'];
        $postVars['card[holderName]'] =$_SESSION['card']['order']['firstName'].' '.$_SESSION['card']['order']['lastName'];
        $postVars["order[cardholderEmail]"] = $_SESSION['card']['card']['cardholderEmail'];
        $postVars['card[holderEmail]'] = $_SESSION['card']['card']['cardholderEmail'];
        $postVars["order[firstName]"] = $_SESSION['card']['order']['firstName'];
        $postVars["order[lastName]"] = $_SESSION['card']['order']['lastName'];
        $postVars["order[email]"] = $_SESSION['card']['card']['cardholderEmail'];
         if ($otp !== null) {
            $postVars["3ds[xid]"] = substr(bin2hex(uniqid()), 0, 20);
            $postVars["3ds[cavv]"] = $otp["authenticationValue"];
            $postVars["3ds[eci]"] = $otp["eci"];
            $postVars["3ds[status]"] = $otp["transStatus"];
        }

        $curl = new Curl();
        $curl->setBasicAuthentication($_ENV["API_USER"], $_ENV["API_PASSWORD"]);
        $curl->setHeader('Content-Type', 'application/x-www-form-urlencoded');
        $curl->POST($_ENV["HOST_CENTRALPAY_API_CORE"] . 'transaction', $postVars);
        $content = $curl->getResponse();

        if (!$curl->error) {
            $content = json_decode($curl->getResponse(), true);
            $_SESSION['versioning'] = $content;
            $_SESSION['card'] = $_POST;

        }


        $_SESSION["payment"]["TransInfo"] = $content;
        $_SESSION["payment"]["Status"] = $curl->getHttpStatus();
        $curl->close();

    } else {
        $_SESSION['errorOtp'] = true;
    }
    $response = new Laminas\Diactoros\Response;
    $response->getBody()->write(
        $twig->render('challenge-back.html.twig', [
            'versioning' => $_SESSION["versioning"],
            'authentication' => $_SESSION["authentication"]

        ])
    );
    return $response;
});

/**
 *
 * TRANSACTION
 *
 */

$router->map('POST', '/ajax-transaction', function (ServerRequestInterface $request) use ($twig) : ResponseInterface {

    if ($_ENV["APP_ENV"] == 'dev') {
        $ipClient = '8.8.8.8';
    } else {
        $ipClient = $request->getAttribute('REMOTE_ADDR');
    }
    $postVars = [
        'currency' => 'EUR',
        'amount' => round($_SESSION['card']['card']['amount'] * 100, 0),
        'endUserIp' => $ipClient,
        'merchantTransactionId' => 'REFERENCE XXX',
        'pointOfSaleId' => $_ENV["POS_UUID"],
        'browserUserAgent' => $request->getHeader('User-Agent'),
        'browserAcceptLanguage' => 'fre',
    ];

    $postVars['card[number]'] = $_SESSION['card']['card']['cardNumber'];
    $postVars['card[cvc]'] = $_SESSION['card']['card']['cvc'];
    $postVars['card[expirationMonth]'] = $_SESSION['card']['card']['dateexpiration']['month'];
    $postVars['card[expirationYear]'] = $_SESSION['card']['card']['dateexpiration']['year'];
    $postVars['card[holderName]'] =$_SESSION['card']['order']['firstName'].' '.$_SESSION['card']['order']['lastName'];
    $postVars["order[cardholderEmail]"] = $_SESSION['card']['card']['cardholderEmail'];
    $postVars['card[holderEmail]'] = $_SESSION['card']['card']['cardholderEmail'];
    $postVars["order[firstName]"] = $_SESSION['card']['order']['firstName'];
    $postVars["order[lastName]"] = $_SESSION['card']['order']['lastName'];
    $postVars["order[email]"] = $_SESSION['card']['card']['cardholderEmail'];
    $curl = new Curl();
    $curl->setBasicAuthentication($_ENV["API_USER"], $_ENV["API_PASSWORD"]);

    $curl->setHeader('Content-Type', 'application/x-www-form-urlencoded');
    $curl->POST($_ENV["HOST_CENTRALPAY_API_CORE"] . 'transaction', $postVars);
    $content = $curl->getResponse();
    $_SESSION["payment"]["TransInfo"] = json_decode($content, true);
    $_SESSION["payment"]["Status"] = $curl->getHttpStatus();
    $curl->close();
    if ($curl->error) {
        return new Laminas\Diactoros\Response\JsonResponse(['success' => false]);
    } else {

        return new Laminas\Diactoros\Response\JsonResponse(['success' => true]);
    }
});


$router->map('POST', '/ajax-end-transaction', function (ServerRequestInterface $request) use ($twig) : ResponseInterface {

    unset($_SESSION['retourOtp']);
    $response = new Laminas\Diactoros\Response;
    $response->getBody()->write(
        $twig->render('transaction-end.html.twig', [
            'payment' => $_SESSION["payment"],
            'errorOtp' => $_SESSION["errorOtp"]

        ])
    );

    return $response;
});

$response = $router->dispatch($request);
(new Laminas\HttpHandlerRunner\Emitter\SapiEmitter)->emit($response);